#include <stdbool.h>
#include <stddef.h>
#include <sys/mman.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <stdint.h>

#include "align.h"
#include "chunk.h"
#include "chunk_manager.h"

int add_new_block(chunk_t* chunk, size_t size, void *prev, void *next, bool is_used)
{
	static int	block_id = 0;
	void		*new_block;
	chunk_t		previous_chunk;

	// TODO check with getrlimit()

	new_block = mmap(NULL, size + ALIGN_MARGING + CHUNK_SIZE, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);

	chunk_init(chunk, size + ALIGN_MARGING - get_align_increment(new_block + CHUNK_SIZE), block_id, prev, new_block, next, is_used);
	
	if (prev != NULL)
	{
		chunk_read(prev, &previous_chunk);
		previous_chunk.next = chunk->current;
		chunk_write(&previous_chunk);
	}

	block_id++;

	return 0;
}

void destroy_block(chunk_t *chunk)
{
    chunk_t chunk_prev, chunk_next, *first_chunk = chunk, *last_chunk = chunk;
    
    if (chunk->prev != NULL)
    {
        chunk_read(chunk->prev, &chunk_prev);
        if (chunk_prev.block_id == chunk->block_id && chunk_prev.is_used == false)
            chunk_merge(&chunk_prev, chunk);
        first_chunk = &chunk_prev;
    }
    if (chunk->next != NULL)
    {
        chunk_read(chunk->next, &chunk_next);
        if (chunk_next.block_id == chunk->block_id && chunk_next.is_used == false)
            chunk_merge(chunk, &chunk_next);
        last_chunk = &chunk_next;
    }

    if (last_chunk != chunk)
    {
        last_chunk->prev = chunk->prev;
        chunk_write(last_chunk);
    }

    chunk->is_used = false;
    chunk_write(chunk);

    if (first_chunk != chunk)
    {
        first_chunk->next = chunk->next;
        chunk_write(first_chunk);
    }

	munmap(chunk->current, get_physical_size(chunk));
}